package sa;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.sidiff.difference.matcher.BaseMatcher;

import SA.Architecture;
import SA.SAPackage;

/**
 * Basically NamedElement Matcher, special handling of Root Element
 * (Architecture)
 * 
 * @author kehrer
 */
public class SaMatcher extends BaseMatcher {

	public static final String KEY = "SaMatcher";

	/**
	 * Initialize matcher and start matching.
	 */
	public SaMatcher() {
		super();
	}

	@Override
	protected boolean isCorresponding(EObject elementA, EObject elementB) {
		assert (elementA != null && elementB != null) : "One of the elements to check for correspondence is null!";

		// type should be identical in order to correspond
		if (elementA.eClass() != elementB.eClass()) {
			return false;
		}

		// None of the elements must be already in a correspondence
		if (isCorresponding(elementA) || isCorresponding(elementB)) {
			return false;
		}

		// "Singleton" elements
		if (elementA instanceof Architecture) {
			return true;
		}

		// Named Elements
		// Check for attribute "name" and its values equality
		EStructuralFeature attrName = elementA.eClass().getEStructuralFeature("name");
		if (attrName != null && attrName instanceof EAttribute) {
			Object nameA = elementA.eGet(attrName);
			Object nameB = elementB.eGet(attrName);

			if (nameA != null && nameB != null) {
				return nameA.equals(nameB);
			}
		}

		// We should never get here..
		assert (false);
		return false;
	}

	@Override
	public String getName() {
		return "SA Matcher";
	}

	@Override
	public String getKey() {
		return KEY;
	}

	@Override
	public String getDocumentType() {
		return SAPackage.eNS_URI;
	}

	@Override
	public boolean canComputeReliability() {
		return false;
	}
}
